//+------------------------------------------------------------------+
//|                                           Indicator Template.mq5 |
//|                                       Copyright 2020, Dale Woods |
//|                                      https://www.theforexguy.com |
//+------------------------------------------------------------------+

#define APP_VERSION 1.00
//#define DRAW_BUFFERS    2

#property version  "1.0"
#property copyright "Copyright 2020, Dale Woods"
#property link      "https://www.theforexguy.com"
#property strict

#property indicator_separate_window

#define EMPTY_BUFFER_VALUE EMPTY_VALUE

#property indicator_buffers 3
#property indicator_plots 2

#property indicator_width1 6
#property indicator_style1 STYLE_SOLID
#property indicator_type1  DRAW_COLOR_HISTOGRAM
#property indicator_label1 "Black Line"
#property indicator_color1 Black


#property indicator_width2 2
#property indicator_style2 STYLE_SOLID
#property indicator_type2  DRAW_LINE
#property indicator_label2 "Black Line"
#property indicator_color2 Red
   

//+------------------------------------------------------------------+
//|   DECLARE GLOBALS                                                |
//+------------------------------------------------------------------+
//#include "includes\Globals.mqh";
#include "DazLib\DataTypes.mqh"
#include "Includes\RejectionData.mqh"
#include "Includes\BodyStrength.mqh"

#include "Includes\ColorGradient.mqh"

bool g_canInitApp = false, g_AppReady = false, g_dataBuilt = false;
int g_candleScanLimit = 0;

double black_line[], red_line[], histogram_colors[];
RejectionData* rejectionData;
BodyStrength* bodyData;

// Must be a child of IndicatorAPI
IndicatorAPI* mainApp = rejectionData;


//+------------------------------------------------------------------+
//|   INPUT SETTINGS                                                 |
//+------------------------------------------------------------------+
input enum_HistoyScanBack scanLimit = AUTO;



//+------------------------------------------------------------------+
//|   INIT                                                           |
//+------------------------------------------------------------------+
int OnInit() {    
   
   // Get requested scan limit from user
   g_candleScanLimit = GetCandleScanLimit(scanLimit, _Period);
   
   SetBuffers();   
   
   // For iCustom calls , try not to use timer
   if( !NoTimerLoad() ) {
      EventSetMillisecondTimer(1);
   }
   
   return(INIT_SUCCEEDED);
}







//+------------------------------------------------------------------+
//| DEINIT                                                           |
//+------------------------------------------------------------------+
void OnDeinit(const int reason) {
   Comment("");   
   Memory::Release(rejectionData);
   Memory::Release(bodyData);
}




/*
   Need to use OnTimer to check if the data is ready before trying to load the indicator
*/
void OnTimer() { 
   
   EventKillTimer(); 
         
   const int recallTimer = 100;
   
   //pDebug(StringFormat("Called g_canInitApp = %d | g_AppReady = %d | g_dataBuilt = %d", g_canInitApp, g_AppReady, g_dataBuilt));   
  // canInit = copied > 0 && bars > 0 && AreBuffersReady(30);
      
   if( !g_canInitApp ) { 
      
      // Use this inside Instrument metrics
      double tmp[];
      const int copied = CopyClose(Symbol(), Period(), 0, 1, tmp);
      
      #ifndef __MQL4__
         g_canInitApp = SymbolIsSynchronized(Symbol()) && 
                        SeriesInfoInteger(Symbol(),0,SERIES_SYNCHRONIZED) && 
                        copied > 0;      
      #else
         g_canInitApp = copied > 0;
      #endif 
     
      EventSetMillisecondTimer(recallTimer);
      return; 
   }
   
   // Load objects and start data loading
   else if( g_canInitApp && !g_AppReady )  {           
      InitApp();        
      EventSetMillisecondTimer(recallTimer);
      return;       
   }
   
   // First time data build
   else if( g_canInitApp && g_AppReady && !g_dataBuilt ) {
      
      // Check chart histroy is loaded and price action data is ready
      #ifndef __MQL4__
         int barsLoaded = mainApp.GetBarsCount();
         int barsGoal = MathMin( mainApp.CalculateEstimatedServerBars(), mainApp.GetRequiredCandleCount() ); 
         
         if( !mainApp.IsReady(barsGoal) ) { 
            printf("COMPILING PRICE ACTION DATA: %s %d/%d bars loaded", Symbol(), barsLoaded, barsGoal);
            
            EventSetMillisecondTimer(recallTimer);
            return;
         }
      #endif
      
      // Check internal objects are loaded and ready
      if( !mainApp.IsReady() ) { 
         EventSetMillisecondTimer(recallTimer);
         return;
      }
      
      // Reset the candle scan limit, in the case there was no history on init - the IndicatorAPI class will have a <= 0 Value stored here
      const int desiredScanBackLimit = GetCurrentScanLimit();      
      mainApp.SetMaxCandlesToScan(desiredScanBackLimit);      
      int newLimit = mainApp.GetScanBackLimit();
      
      if( newLimit <= 0 ) {          
         EventSetMillisecondTimer(recallTimer);
         return; 
      }      
            
      BuildData(newLimit); 
      
      // Tell the IndicatorAPI we already built data for all the bars
      mainApp.SetCalculated( Bars( Symbol(), Period()) );          
      IndicatorLoop(newLimit);
      
      g_dataBuilt = true;
      
      printf("Loaded");
   }
}




// Init code for when chart is ready
void InitApp() {

   // Get scan back limit
   int candleScanLimit = GetCurrentScanLimit();

   // Load current time frame instance (main object)
   rejectionData = new RejectionData(3, candleScanLimit, _Symbol, _Period);   
   bodyData = new BodyStrength(5, candleScanLimit, _Symbol, _Period);
   
   mainApp = rejectionData;
      
   g_AppReady = true;
}




// User requested limit vs available bars.
int GetCurrentScanLimit() {
   return MathMin( g_candleScanLimit, Bars(_Symbol, _Period));
}


void BuildData(const int newSize) {   
      
   mainApp.SetMaxCandlesToScan(newSize);
   mainApp.BuildData();
   
   bodyData.SetMaxCandlesToScan(newSize);
   bodyData.BuildData();
}



//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
int OnCalculate(
   const int rates_total,
   const int prev_calculated,
   const datetime &time[],
   const double &open[],
   const double &high[],
   const double &low[],
   const double &close[],
   const long &tick_volume[],
   const long &volume[],
   const int &spread[]
) {     
   static bool clearedBuffers = false;
   static bool firstRun = true;
   static datetime lastCheckedCandleTime = 0;
  
  
   // Buffers not ready
   if( ArraySize(black_line) < 1 ) { return 0; }
   
   // Stop graphical glitches
   if( !clearedBuffers ) {
    //  EmptyBuffers();
      clearedBuffers = true;
   }
   
   
   // Check if this is the first call of OnCalculate
   if( firstRun ) { 
      OnFirstRun();      
      firstRun = false;
   }
   
   // App is ready to receive ticks
   if( !g_canInitApp || !g_AppReady || !g_dataBuilt ) { return rates_total; }
   
   // New candle check
   ArraySetAsSeries(time, true);
   
   if( time[0] != lastCheckedCandleTime ) {
      OnNewCandle();
      lastCheckedCandleTime = time[0];
   }
     
   //check if prev calcualted has been reset
   if( mainApp.ShouldRebuildData(prev_calculated) )  { 
      
      int newLimit = GetCurrentScanLimit();
      EmptyBuffers();
      BuildData(newLimit);     
      
      //pDebug(StringFormat("Calling IndicatorLoop with %i", newLimit));
      IndicatorLoop(newLimit);
   }
   
   else { 
      int limit = g_candleScanLimit;
      if( prev_calculated == 0 ) { limit = MathMin(rates_total-1, limit); }
      
      else {
         limit = MathMin(   
            rates_total-prev_calculated,     //onCalculate counter
            limit-1                          // User setting               
         );
      }
      
      IndicatorLoop(limit);
   }
   
   return(rates_total);
}



void OnFirstRun() {

   if( NoTimerLoad() ) {
      g_canInitApp   = true;
      g_AppReady     = true;      
      g_dataBuilt    = true;      
      InitApp();         
      mainApp.SetMaxCandlesToScan( GetCurrentScanLimit() );   
      mainApp.SetShouldRebuildData(); // Force next should rebuild data call to be true
   }

}

void OnNewCandle() {
   ClearBuffersAt(0);     
}



//+------------------------------------------------------------------+
//|   INDICATOR LOOP                                                 |
//+------------------------------------------------------------------+ 



/** 
 * 	Call OnCalculate with resetted params
**/
void RestartOnCalculate() {
   mainApp.ShouldRebuildData();
   EmptyBuffers();   
   IndicatorLoop( GetCurrentScanLimit() );
   ChartRedraw();
}







//+------------------------------------------------------------------+
//|   ON CHART EVENT                                                 |
//+------------------------------------------------------------------+      
void OnChartEvent(
   const int id,           // Event identifier  
   const long& lparam,     // Event parameter of long type
   const double& dparam,   // Event parameter of double type
   const string& sparam    // Event parameter of string type
) {   
   
   bool refreshChart = false;
   
   if(id == CHARTEVENT_KEYDOWN) {   
      OnKeyPress( (const int) lparam );
      refreshChart = true;
   }
   
   if( refreshChart ) {
      RestartOnCalculate();
   }
}//onChartEvent



//+------------------------------------------------------------------+
//|   ON KEY PRESS EVENT HANDLE                                      |
//+------------------------------------------------------------------+
void OnKeyPress(const int asciInt) {
   switch(asciInt) { 
      case 68: {
         break;
      } // "d" key pressed    
   }  
}




bool NoTimerLoad() {
   return false;
}



//+------------------------------------------------------------------+
//|   INDICATOR BUFFER FUNCTIONS                                     |
//+------------------------------------------------------------------+ 
void SetBuffers() {
   
   ArraySetAsSeries(black_line, true);   
   ArraySetAsSeries(red_line, true);     
   ArraySetAsSeries(histogram_colors, true); 
      
   //PlotIndexSetDouble(i, PLOT_EMPTY_VALUE, EMPTY_BUFFER_VALUE);      
   //PlotIndexSetInteger(i, PLOT_DRAW_BEGIN, g_candleScanLimit-1);
  // }
   
      
   //inside candle buffers
   SetIndexBuffer(0,black_line); 
   SetIndexBuffer(2,red_line); 
     
   SetIndexBuffer(1,histogram_colors,INDICATOR_COLOR_INDEX);
   
   
   //Assign the color indexes array with indicator's buffer
   setPlotColor(0);
}   



void IndicatorLoop( const int startAtIndex ) {
   double multiplier= 2 * (M_PI/60);
   
   mainApp.Update();
   
   for(int i=startAtIndex; i>=0; i--) {
         
    //  black_line[i]   =  rejectionData.Get(i) / SymbolInfoDouble(_Symbol, SYMBOL_POINT);
    //  black_line[i]   = Math::RandDouble(1,100);
      // red_line[i]    = bodyData.Get(i) / SymbolInfoDouble(_Symbol, SYMBOL_POINT);
      
      black_line[i]  = sin(i * multiplier);
      red_line[i] = sin(i * multiplier - 0.5);
      //printf("black_line[i] = %f", black_line[i]);
     
      histogram_colors[i] =  getPlotColor(black_line[i], -1.0, 1.0);
       
   }//for
   ChartRedraw();
}




   
   
   
void EmptyBuffers() { 
   
   ArrayInitialize(black_line, EMPTY_VALUE);
   ArrayInitialize(red_line, EMPTY_VALUE);
}   



void ClearBuffersAt(const int index) {
   
   // Try to stop annoying drawing glitches on new candles
   if( ArraySize(black_line) > index ) { 
      black_line[index] = EMPTY_VALUE;
      red_line[index] = EMPTY_VALUE;
   }
}



void setPlotColors(int plot) {
   const int gradCount = 60;
   PlotIndexSetInteger(plot,PLOT_COLOR_INDEXES,gradCount); //Set number of colors
   
   ColorGradient gradient(gradCount);
   gradient.SetColorsMql(clrWhite, clrBlack);
   gradient.SetColorsMql(clrBlue, clrRed);
  
   // gradient.SetColorsMql(clrRed, clrGray, clrBlue);
   
   for(int i=0;i<=gradCount - 1;i++) {
      PlotIndexSetInteger(plot,PLOT_LINE_COLOR,i, gradient.GetColorAt(i));
   }
   
}
//+------------------------------------------------------------------+
//|  Get index of the color                                          |
//+------------------------------------------------------------------+
/*
*       The function return the color index
*       The first parameter is the current value of the indicator
*       The second parameter is the minimal value of the indicator
*       The third parameter is the maximal value of the indicator
*/
double getPlotColor(double current,double min,double max) {
   int val = 0 ;
   
   if( min < 0 ) { 
      double shift = fabs(min);
      val = (int) (50 * Math::NormalizeValue(current+shift,0,max+shift));
   }
   
   else {
      val = (int) (50 * Math::NormalizeValue(current,min,max));
   }
   
   return (double) MathMin( MathMax(0, val), 49);
}
///////


function 