//+------------------------------------------------------------------+
//|                                                ColorGradient.mqh |
//|                                      Copyright 2018, Dale Woods. |
//|                                      https://www.theforexguy.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2018, Dale Woods."
#property link      "https://www.theforexguy.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class ColorGradient {
   private:
      uint 
         _startColor,
         _midColor,
         _endColor,
         _totalSteps,
         _phaseSteps,
         _midPos,
         _colorCount,
         
         _startRed,
         _endRed,
         
         _startGreen,
         _endGreen,
         
         _startBlue,
         _endBlue,
         
         _midRed,
         _midGreen,
         _midBlue;

      
      double
         _exponentialCoeff;  
         
      bool 
         _useExponentialGradient;
         
      uint GetRedValue(const uint _RGBVal) const;
      uint GetGreenValue(const uint _RGBVal) const;
      uint GetBlueValue(const uint _RGBVal) const;
      
      uint GetColorInRange(const int stepNo, const uint from, const uint to) const;  
      
      double CalculatePosition(const int stepNo, const uint stepCount, const bool gradientNegative = false) const;
      double CalculateSteps(const int stepNo, const uint stepCount, double coeff) const;
      
      uint ToRGB(const color clr) const;
      uint ToRGB(const uint r, const uint g, const uint b) const;
      color ToColor(const uint r, const uint g, const uint b) const;
      
      void SplitColors();
              
   public:
      ColorGradient(
         uint totalSteps
      );
            
      ~ColorGradient();
      
      void SetSteps(const uint setTo);
      void SetExponentialGradient(const double coeff);
      
      // 2 Color Gradient
      void SetColorsRGB(const uint start, const uint end);
      void SetColorsMql(const color start, const color end);
      
      // 3 Color Gradient
      void SetColorsRGB(const uint start, const uint mid, const uint end);
      void SetColorsMql(const color start, const color mid, const color end);
      
      uint GetColorAt(const uint step, const bool RGB = false) const;
      
      string To_HexString(uint num) const;
      
};


ColorGradient::ColorGradient(
   uint totalSteps
) {
   _startColor    =  0;
   _endColor      =  0;
   _midColor      = 0;
   SetSteps(totalSteps);
   _exponentialCoeff = 0;
   _useExponentialGradient = false;
   
}


ColorGradient::~ColorGradient() {
}



void ColorGradient::SetSteps(const uint setTo) {
   
   if( setTo == 0 ) { 
      printf("[%s ERROR]: Cannot set steps to zero", __FUNCTION__);
      return;
   }
   
   _totalSteps = setTo;
   _midPos = (int) _totalSteps / 2;
   
   // We want mid pos to be odd number so there is the same count on both the left and right side
   if( _midPos % 2 == 0 ) {
      _midPos++;
      _totalSteps++;
   }
   
   _phaseSteps = (_totalSteps / 2) -1;
}


void ColorGradient::SetExponentialGradient(const double coeff) {
   
   if( coeff == 0 ) { 
      printf("[%s ERROR]: Cannot set Exp Coeff to zero", __FUNCTION__);
      return;
   }
   
   _exponentialCoeff = coeff;
   _useExponentialGradient = true;
}


void ColorGradient::SetColorsRGB(const uint start, const uint end) {
   _startColor = start;
   _endColor = end;
   
   _colorCount = 2;
   
   SplitColors();
}


void ColorGradient::SetColorsMql(const color start, const color end) {
   _startColor = ToRGB(start);// ToRGB(start);
   _endColor = ToRGB(end); //ToRGB(end);
   
   _colorCount = 2;
   
   SplitColors();
}


void ColorGradient::SetColorsRGB(const uint start, const uint mid, const uint end) {

   _startColor = start;
   _midColor = mid;
   _endColor = end;

   _colorCount = 3;
   
   SplitColors();
}


void ColorGradient::SetColorsMql(const color start, const color mid, const color end) {
   
   _startColor = ToRGB(start);// ToRGB(start);
   _midColor = ToRGB(mid);
   _endColor = ToRGB(end); //ToRGB(end);

   _colorCount = 3;
   
   SplitColors();
}





uint ColorGradient::GetRedValue(const uint _RGBVal) const {
   return (_RGBVal & 0x00FF0000) >> 16;
}


uint ColorGradient::GetGreenValue(const uint _RGBVal) const {
   return (_RGBVal & 0x0000FF00) >> 8;
}


uint ColorGradient::GetBlueValue(const uint _RGBVal) const {
   return (_RGBVal & 0x000000FF);
}


color ColorGradient::ToColor(const uint r, const uint g, const uint b) const {
    return (color) ( (b << 16) + (g << 8) + r);
}


uint ColorGradient::ToRGB(const uint r, const uint g, const uint b) const {   
   return ( 0xFF000000 | ( ( r << 16 ) | ( g << 8 ) | b ) );
}


uint ColorGradient::ToRGB(const color clr) const {
   
   uint
      red   = (0x000000FF & clr) << 16 ,
      green = (0x0000FF00 & clr),
      blue  = (0x00FF0000 & clr) >> 16;
      
   uint rgbColor = 0xFF000000 | red | green | blue;
      
   return rgbColor;
   //return (uint) ( 0xFF000000|(uchar(clr>>16)<<16)|(uchar((clr)>>8)<<8)|uchar((clr)>>16) );
}



void ColorGradient::SplitColors() {
   if( _startColor == 0xFF000000 ) { _startColor = 0xFF000001; }

   _startRed      = GetRedValue(_startColor);
   _endRed        = GetRedValue(_endColor);
   
   _startGreen    = GetGreenValue(_startColor);
   _endGreen      = GetGreenValue(_endColor);
   
   _startBlue     = GetBlueValue(_startColor);
   _endBlue       = GetBlueValue(_endColor);
   
   if( _colorCount == 3 ) {
      _midRed     = GetRedValue(_midColor);
      _midGreen   = GetGreenValue(_midColor);
      _midBlue    = GetBlueValue(_endColor);
   }   
}


double ColorGradient::CalculatePosition(const int stepNo, const uint stepCount, const bool gradientNegative = false) const {
   double pos = 0;
   
   if( stepNo == 0 ) { return 0; }
   
   pos = (double) stepNo / (double) stepCount;
   
   if( gradientNegative ) { 
      pos = 1 - pos;
   }
   return pos;
   return MathMax( MathMin(pos, 1), 0);
}



double ColorGradient::CalculateSteps(const int stepNo, const uint stepCount, double expCoff) const {
   if( !_useExponentialGradient ) { return stepNo; }
   
   // Using x exp y only scales the gradient towards one end
   double 
      maxValueSquared = MathPow(stepCount, expCoff),
      currentStepSquared = MathPow(stepNo, expCoff),  
      coeff = currentStepSquared / maxValueSquared;
   
   
   return coeff * stepNo;

}



uint ColorGradient::GetColorInRange(const int stepNo, const uint _8bitStart, const uint _8bitEnd) const {
   uint length = _totalSteps;// - 1.0;
   
   if( _colorCount == 3 ) { 
      length = _phaseSteps;
   }
   
   bool inverted = false;
   
   int range =  (int) _8bitEnd - (int) _8bitStart;
   
   if(range == 0) { return _8bitStart; }
   
   if( range < 0 ) {
      inverted       = true;   
      range = range * -1;
   }
   
   double 
      colorStepVal =  (double) range / (double) length, 
      pos = CalculatePosition( stepNo, length, inverted );
   
   // Calculate linear color change   
   double colorVal = 0;
   
   double steps = CalculateSteps(stepNo, length, _exponentialCoeff);
   
   if( inverted ) {    
     colorVal = _8bitStart - ( colorStepVal * steps );
   }
   
   else {
      colorVal = _8bitStart + (colorStepVal * steps );
   }
   
   // colorVal = pos * range;
   //printf("INVERTED[%d]: (pos %f * range %f) = %f", inverted, pos, range, colorVal);
    
   colorVal = MathRound(colorVal);
   uint colorFinal = (uint) NormalizeDouble(colorVal, 0);
   
   return colorFinal;
}


uint ColorGradient::GetColorAt(const uint step, const bool RGB = false) const {
   
   uint 
      _b = _midBlue,
      _g = _midGreen,
      _r = _midRed;
   
   
   if( _colorCount == 2 ) {   
      _b  = GetColorInRange(step, _startBlue, _endBlue );
      _g  = GetColorInRange(step, _startGreen, _endGreen );
      _r  = GetColorInRange(step, _startRed, _endRed);
   }
   
   
   if( _colorCount == 3 ) { 
      uint pos = step;
      if( step < _midPos ) { 
         _b  = GetColorInRange(pos, _startBlue, _midBlue );
         _g  = GetColorInRange(pos, _startGreen, _midGreen );
         _r  = GetColorInRange(pos, _startRed, _midRed);
      }
      
      if( step > _midPos ) { 
         pos = step - _phaseSteps;
       //  printf("step[%d] = pos[%d]", step, pos);
         _b  = GetColorInRange(pos, _midBlue, _endBlue );
         _g  = GetColorInRange(pos, _midGreen, _endGreen );
         _r  = GetColorInRange(pos, _midRed, _endRed);
      }      
   }
   
   
   if( RGB ) {  
      return ToRGB(_r, _g, _b);
   }
      
   return (color) ToColor(_r, _g, _b);
}


string ColorGradient::To_HexString(uint num) const {
   
   int len=0,k=0;
   char buff[64];
   char __hex [] = { '0', '1', '2', '3', '4', '5', '6', '7',
                     '8', '9' ,'A', 'B', 'C', 'D', 'E', 'F' };
                        
   do
      {
      uint n=num&0xF;
      buff[len] = __hex[n];
      len++;
      num>>=4;
      }
   while(num!=0);
   
   for(;k<len/2;k++)
      {
      buff[k]^=buff[len-k-1];
      buff[len-k-1]^=buff[k];
      buff[k]^=buff[len-k-1];
      }
   return CharArrayToString(buff,0,len);
}